import os
import asyncio
from collections import OrderedDict
from configparser import ConfigParser, ExtendedInterpolation

# from aio.core.exceptions import MissingConfiguration


def gather_config(modules, filename=None):
    config_files = []
    for module in modules:
        path = module.__path__[0]
        app_config = os.path.join(path, filename or "aio.conf")
        if os.path.exists(app_config):
            config_files.append(app_config)
    return config_files


def find_config(app_dir=None, filename=None):
    if not app_dir:
        app_dir = os.getcwd()
    config = os.path.join(app_dir, filename or 'aio.conf')
    if os.path.exists(config):
        return config
    config = os.path.join(app_dir, 'etc', filename or 'aio.conf')
    if os.path.exists(config):
        return config
    config = os.path.join(app_dir, '/etc', filename or 'aio.conf')
    if os.path.exists(config):
        return config
    # raise MissingConfiguration('no configuration file found')


@asyncio.coroutine
def parse_config(config=None, config_files=None, app_dir=None,
                 config_dict=None, config_string=None,
                 modules=None, parser=None, interpolation=None,
                 filename=None):
    parser = parser or ConfigParser(
        interpolation=interpolation or ExtendedInterpolation())
    if config_dict:
        parser.read_dict(config_dict)
    elif config_string:
        parser.read_string(config_string)
    elif config_files:
        for config_file in config_files:
            parser.read_file(open(config))
    elif modules:
        for config_file in gather_config(modules, filename=filename):
            parser.read_file(open(config_file))
    else:
        config = config or find_config(app_dir, filename=filename)
        if config:
            parser.read_file(open(config))
    return parser


class ConfigSection(OrderedDict):

    pass


class Config(OrderedDict):

    pass


def replicate_config(config, test_section=None):
    new_config = Config()
    for section_name, section in config.items():
        if test_section:
            if not test_section(section_name):
                continue
        new_config[section_name] = ConfigSection()
        for option_name, option in section.items():
            new_config[section_name][option_name] = option
    return new_config



import os
import unittest

from aio.testing import aiotest
import aio.config
import aio.testing
from aio.config.tests import (
    example_module, example_module2)

TEST_DIR = os.path.dirname(__file__)

CONFIG_STRING = """
[section1]
bar: 1
foo: 2

[section2]
bar: 3
foo: 4
"""


class AioConfigParseTestCase(unittest.TestCase):

    @aiotest
    def test_parse_config_modules(self):
        """
        this test should parse config from multiple paths
        """
        config = yield from aio.config.parse_config(
            modules=[
                aio.testing, aio.config,
                example_module, example_module2])

        # check interpolation between the module conf
        self.assertEqual(
            "default",
            config['more_settings']['example_option2'])
        self.assertEqual(
            "default",
            config['even_more_settings']['example_option'])

    @aiotest
    def test_parse_config_app_dir(self):
        """
        this test should find and parse "aio.conf" in the app_dir
        """
        app_dir = os.path.join(
            TEST_DIR, "resources")
        config = yield from aio.config.parse_config(app_dir=app_dir)
        self.assertEqual(config.sections(), ["section1"])
        self.assertEqual(config["section1"]["result"], "1")

    @aiotest
    def test_parse_custom_config(self):
        """
        this test should parse a custom config file
        """
        configfile = os.path.join(
            TEST_DIR, "resources", "test-1.conf")
        config = yield from aio.config.parse_config(configfile)
        self.assertEqual(config.sections(), ["foo", "bar"])
        self.assertEqual(config["foo"]["bar"], "1")
        self.assertEqual(config["bar"]["foo"], "baz")

    @aiotest
    def test_config_string(self):
        config = yield from aio.config.parse_config(
            config_string=CONFIG_STRING)
        self.assertEqual(config.sections(), ["section1", "section2"])
        self.assertEqual(config["section1"]["foo"], "2")
        self.assertEqual(config["section2"]["bar"], "3")


class AioConfigFinderTestCase(unittest.TestCase):

    def test_find_config(self):
        """
        this test should find "aio.conf" in the app_dir
        """
        app_dir = os.path.join(
            TEST_DIR, "resources")
        self.assertEqual(
            aio.config.find_config(app_dir),
            os.path.join(
                app_dir, 'aio.conf'))

    def test_find_config_etc(self):
        """
        this test should find "aio.conf" in the etc
        """
        app_dir = os.path.join(
            TEST_DIR, "resources", "sub")
        self.assertEqual(
            aio.config.find_config(app_dir),
            os.path.join(
                app_dir, "etc", 'aio.conf'))


class AioConfigGatherTestCase(unittest.TestCase):

    def test_gather_config(self):

        from aio.config.tests import (
            example_module, example_module2)

        config_files = aio.config.gather_config(
            [aio.testing, aio.config,
             example_module, example_module2])

        expected = [
            os.path.join(m.__path__[0], "aio.conf")
            for m in [example_module, example_module2]]

        self.assertEqual(config_files, expected)

    def test_gather_config_custom_filename(self):

        config_files = aio.config.gather_config(
            [aio.testing, aio.config,
             example_module, example_module2])

        expected = [
            os.path.join(m.__path__[0], "aio.conf")
            for m in [example_module, example_module2]]

        self.assertEqual(config_files, expected)
